/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *    Brutus header file for Brutus CORBA utility functions.
 *    Copyright (C) 2005-2007 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA 02111-1307 USA
 */


#ifndef _BRUTUS_CORBA_H_
#define _BRUTUS_CORBA_H_

#include <orbit/orbit.h>
#include <glib/gtypes.h>

#include <idl_output/types.h>
#include <idl_output/ISequentialStream.h>

G_BEGIN_DECLS

/*
 * This method will return an ORB which is so initialized
 * that the remote Brutus Logon object can be resolved
 * with resolve_initial_references().
 *
 * argc and argv are command line arguments with other
 * options that must be given to the ORB in ORB_init().
 *
 * argc                  : ORB command line argument count
 *
 * argv                  : ORB command line arguments
 *
 * logon_init_ref        : String to pass to resolve_initial_references() for BrutusLogOn::
 *
 * logon_obj_server_key  : Object key given to BrutusLogOn:: object at the server
 *
 * server_host_name      : Hostname or IP address of remote Brutus server
 *
 * server_port           : Port number where the remote Brutus Logon object is exposed
 *
 * orb_name              : Name of return ORB or empty string
 *
 * Return value          : Initialized ORB or CORBA::ORB::_nil()
 *
 */
extern CORBA_ORB brutus_create_orb(int argc,
				   const char **argv,
				   const char *logon_init_ref,
				   const char *logon_obj_server_key,
				   const char *server_host_name,
				   const guint16 server_port,
				   const char *orb_name,
				   CORBA_Environment *ev);

/* Returns the Root Poa */
extern PortableServer_POA brutus_get_root_poa(const CORBA_ORB Orb,
					      CORBA_Environment *ev);

/* Creates a child POA - not activated */
extern PortableServer_POA brutus_create_poa(const char *Name,
					    PortableServer_POAManager PoaManager,
					    PortableServer_POA ParentPOA,
					    CORBA_Environment *ev);

/* Creates an activated child POA */
extern PortableServer_POA brutus_create_activated_poa(PortableServer_POA ParentPOA);

/* Releases the object and frees associated memory  */
extern void brutus_free_orbit_object(CORBA_Object ref,
				     PortableServer_POA poa,
				     CORBA_Environment *ev);

/* Deactivates and frees the POA */
extern void brutus_free_poa(PortableServer_POA poa,
			    CORBA_Environment *ev);

/* Deactivates the POA so that no more requests are served
 * but does not free it */
extern void brutus_deactivate_poa(PortableServer_POA poa,
				  CORBA_Environment *ev);

/* Convets a stringified IOR read from "filename" into a CORBA object.
 * Returns CORBA_OBJECT_NIL on error */
extern CORBA_Object brutus_object_ref_from_file(CORBA_ORB orb,
						const gchar *filename,
						CORBA_Environment *ev);

/*
 * Will release *object remotely and locally and unconditionally
 * assign CORBA_OBJECT_NIL to *object
 */
extern void brutus_release_object(const BRUTUS_BrutusTC type_code,
				  CORBA_Object *object,
				  PortableServer_POA poa, // ignored for MAPI wrapper object
				  CORBA_Environment *ev);

/*
 * Will free the contents of a string sequence.
 */
extern void brutus_free_seq_string_content(BRUTUS_seq_string *strings);


/*
 * Will read len octets from stream observing limits on GIOP receive
 * buffer size.
 */
extern BRUTUS_BRESULT brutus_read_stream(BRUTUS_ISequentialStream stream,
					 BRUTUS_seq_octet **data,
					 CORBA_unsigned_long len,
					 CORBA_Environment *ev);


G_END_DECLS

#endif // _BRUTUS_CORBA_H_
