/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *    Brutus header file for Brutus MAPI utility functions.
 *    Copyright (C) 2005-2007 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA 02111-1307 USA
 */

#ifndef _BRUTUS_MAPI_H_
#define _BRUTUS_MAPI_H_

#include <idl_output/types.h>
#include <idl_output/return_codes.h>
#include <idl_output/IMAPITable.h>
#include <idl_output/IMAPIProp.h>
#include <idl_output/IMAPISession.h>
#include <idl_output/IMsgStore.h>
#include <idl_output/IMAPIFolder.h>

G_BEGIN_DECLS

#define BRUTUS_IMAPITABLE_ROW_COUNT_MAX (20)

/*
 * Converts a single stringified ENTRYID into a sequence of ENTRYID.
 * Will return TRUE on success, FALSE otherwise.
 */
extern gboolean brutus_string_to_seq_entryid(const char *uid,
					     BRUTUS_seq_ENTRYID *entryids);

/*
 * Converts a single ENTRYID into a sequence of ENTRYID.
 * Will return TRUE on success, FALSE otherwise.
 */
extern gboolean brutus_entryid_to_seq_entryid(const BRUTUS_ENTRYID *eid,
					      BRUTUS_seq_ENTRYID *entryids);

/*
 * Will free the content of an ENTRYID sequence.
 */
extern void brutus_free_seq_entryid_content(BRUTUS_seq_ENTRYID *entryids);


/*
 * Converts an array of uids to a sequence of ENTRYID.
 * Will return TRUE on success, FALSE otherwise.
 */
extern gboolean brutus_uid_array_to_seq_entryid(const GPtrArray *uids,
						BRUTUS_seq_ENTRYID *entryids);

/*
 * Converts an GList of uids to a sequence of SBinary
 * Will return TRUE on success, FALSE otherwise.
 */
extern gboolean brutus_uid_glist_to_seq_entryid(GList *uids,
						BRUTUS_seq_ENTRYID *entryids);

/*
 * Will free the content of an SBinaryArray.
 */
extern void brutus_free_sbinaryarray_content(BRUTUS_SBinaryArray *bins);

/*
 * Converts an GList of uids to a sequence of SBinary.
 * Will return TRUE on success, FALSE otherwise.
 */
extern gboolean brutus_uid_glist_to_sbinaryarray(GList *uids,
						 BRUTUS_SBinaryArray *bins);

/*
 * Will create a new MAPI folder and all non-existing parent folders, much
 * like 'mkdir -p'.
 *
 * "name" _must_ be of the format
 *
 *     foo/bar/whatever
 *
 * No leading or trailing '/' allowed. "name" will be modified by this
 * function. Use a string_dup'ed value if the string is important to you.
 *
 * "container_class" might be NULL or it will otherwise be used as the value
 * of the PR_CONTAINER_CLASS property.
 *
 * Will return TRUE if all folders was successfully created, FALSE otherwise.
 */
extern gboolean brutus_create_mapi_folder(BRUTUS_IMAPIFolder parent_folder,
					  const char *container_class,
					  char *name);

/*
 * Will open a subfolder by PR_DISPLAY_NAME. Will return CORBA_OBJECT_NIL
 * on error.
 */
extern BRUTUS_IMAPIFolder brutus_open_folder_by_name(BRUTUS_IMAPIFolder parent_folder,
						     const char *display_name);

/*
 * Will copy the content of src to dest
 */
extern void brutus_copy_spropvalue(BRUTUS_SPropValue *dest,
				   const BRUTUS_SPropValue *src);

/*
 * Will free the content of the SPropValue
 */
extern void brutus_free_spropvalue_content(BRUTUS_SPropValue *pv,
					   PortableServer_POA poa);

/*
 * Will walk and free the content of the SPropValue sequence
 */
extern void brutus_free_seq_spropvalue_content(BRUTUS_seq_SPropValue *prop_values,
					       PortableServer_POA poa);

/*
 * Will walk and free the content of the address list
 */
extern void brutus_free_addrlist_content(BRUTUS_ADRLIST *addr_list,
					 PortableServer_POA poa);

/*
 * Returns TRUE if the ENTRYID is shortterm, FALSE otherwise
 */
extern gboolean brutus_entryid_is_shortterm(const BRUTUS_ENTRYID *Eid);

/*
 * Will open the message and return one propvalue from it or NULL in error.
 */
extern BRUTUS_SPropValue *brutus_open_object_get_one_prop(BRUTUS_IMAPISession Session,
							  BRUTUS_IMAPIFolder Folder,
							  BRUTUS_ENTRYID *Eid,
							  const BRUTUS_BDEFINE PropTag);

/*
 * Will return a valid reference or CORBA_OBJECT_NIL
 */
extern BRUTUS_IMessage brutus_open_message(BRUTUS_IMAPISession Session,
					   BRUTUS_IMAPIFolder Folder,
					   const char *UID);

/*
 * Will return a stringified entryid
 */
extern char *brutus_entryid_to_string(const BRUTUS_ENTRYID *Eid);

/*
 * Will convert a stringified ENTRYID back to ENTRYID. eid must
 * not be NULL.
 */
extern gboolean brutus_string_to_entryid_internal_alloc(const char *uid,
							BRUTUS_ENTRYID *eid);

/*
 * Will return a pointer to a BRUTUS_ENTRYID. The return value must be freed
 * with CORBA_free() by the caller.
 */
extern BRUTUS_ENTRYID *brutus_string_to_entryid(const char *uid);

/*
 * Will convert a stringified SBinary back to SBinary. bin must
 * not be NULL.
 */
extern gboolean brutus_string_to_sbinary_internal_alloc(const char *uid,
							BRUTUS_SBinary *bin);

/*
 * Will return a pointer to a BRUTUS_SBinary. The return value must be 
 * freed with CORBA_free() by the caller.
 */
extern BRUTUS_SBinary *brutus_string_to_sbinary(const char *uid);

/*
 * Will return TRUE if they are equal, FALSE otherwise
 */
extern gboolean brutus_entryid_cmp(gconstpointer a,
				   gconstpointer b);

/*
 * Will return the parent folder or CORBA_OBJECT_NIL in error
 */
extern BRUTUS_IMAPIFolder brutus_get_parent_folder(BRUTUS_IMAPISession Session,
						   BRUTUS_IMAPIProp Object);

/*
 * Will return a copy of a BRUTUS_ENTRYID
 */
extern BRUTUS_ENTRYID *brutus_entryid_dup(const BRUTUS_ENTRYID *orig);

/*
 * Converts a binary BLOB to a BRUTUS_ENTRYID
 */
extern void brutus_sbinary_to_entryid(const BRUTUS_SBinary *Bin,
				      BRUTUS_ENTRYID **Eid);

/*
 * Will return PropTag property as a string or NULL in error
 */
extern char *brutus_get_property_as_string(BRUTUS_IMAPIProp Object,
					   const BRUTUS_BDEFINE PropTag);

/*
 * Will return a GUID string or NULL in error
 */
extern char *brutus_get_guid_str(BRUTUS_IMAPIProp Object,
				 const BRUTUS_BDEFINE PropTag);

/*
 * Will return a BRUTUS_ENTRYID for the given proptag or NULL in error
 */
extern BRUTUS_ENTRYID *brutus_get_entryid(BRUTUS_IMAPIProp Object,
					  const BRUTUS_BDEFINE PropTag);

/*
 * Will return a stringified version of the value. NULL is
 * returned for values with length zero or if the property
 * type is unknown. Caller must free the returned string.
 */
extern char *brutus_print_spropvalue(const BRUTUS_SPropValue *PropValue);

/*
 * Will return the index of the queried property tag in
 * the given row or -1 if not found.
 */
extern CORBA_long brutus_get_srow_index(const BRUTUS_SRow *row,
					const BRUTUS_BDEFINE PropTag);

/*
 * Will return the index of the queried property tag in
 * the given SPropValue array or -1 if not found.
 */
extern CORBA_long brutus_get_spropvalue_index(const BRUTUS_seq_SPropValue *props,
					      const BRUTUS_BDEFINE PropTag);

/*
 * This is the Brutus equivalent of HrQueryAllRows()
 */
extern BRUTUS_BRESULT brutus_QueryAllRows(BRUTUS_IMAPITable ptable,
					  BRUTUS_SPropTagArray *ptags,
					  BRUTUS_SRestrictionContainer pres,
					  BRUTUS_SSortOrderSet *psort,
					  CORBA_long crowsMax,
					  BRUTUS_SRowSet **pprows,
					  CORBA_Environment *ev);

/*
 * Will set the cursor at the beginning of the table and 
 * generally prepare the table for a call to QueryRows()
 */
extern BRUTUS_BRESULT brutus_PrepareTable(BRUTUS_IMAPITable ptable,
					  BRUTUS_SPropTagArray *ptags,
					  BRUTUS_SRestrictionContainer pres,
					  BRUTUS_SSortOrderSet *psort,
					  CORBA_Environment *ev);
/*
 * Will call QueryRows() to get a number of rows as defined by crowsMax 
 */
extern BRUTUS_BRESULT brutus_QuerySomeRows(BRUTUS_IMAPITable ptable,
					   CORBA_long crowsMax,
					   BRUTUS_SRowSet **pprows,
					   CORBA_Environment *ev);

/*
 * Will get the total number of rows in the table
 */
extern BRUTUS_BRESULT brutus_QueryRowCount(BRUTUS_IMAPITable ptable,
					   CORBA_unsigned_long *row_count,
					   CORBA_Environment *ev);

/*
 * This is the Brutus equivalent of HrGetOneProp()
 */
extern BRUTUS_BRESULT brutus_GetOneProp(BRUTUS_IMAPIProp MapiProp,
					const BRUTUS_BDEFINE PropTag,
					BRUTUS_SPropValue **PropValue,
					CORBA_Environment *ev);

/*
 * MsgStore is a reference to the default message store
 * upon return or CORBA_OBJECT_NIL
 */
extern BRUTUS_BRESULT brutus_get_default_message_store(BRUTUS_IMAPISession Session,
						       BRUTUS_IMsgStore *MsgStore,
						       CORBA_Environment *ev);

/*
 * RootFolder is a reference to the root folder in MsgStore
 * upon return or NIL
 */
extern BRUTUS_BRESULT brutus_get_root_folder(BRUTUS_IMsgStore MsgStore,
					     BRUTUS_IMAPIFolder *RootFolder,
					     CORBA_Environment *ev);

/*
 * MsgStore is normally the default message store for the profile.
 * Folder is a reference to the requested Outlook specific folder
 * upon return or NIL. Currently predefined tags are:
 *
 *   BRUTUS_BRUTUS_PR_OUTLOOK_CALENDAR
 *   BRUTUS_BRUTUS_PR_OUTLOOK_CONTACTS
 *   BRUTUS_BRUTUS_PR_OUTLOOK_JOURNAL
 *   BRUTUS_BRUTUS_PR_OUTLOOK_NOTES
 *   BRUTUS_BRUTUS_PR_OUTLOOK_TASKS
 *   BRUTUS_BRUTUS_PR_OUTLOOK_REMINDERS
 *   BRUTUS_BRUTUS_PR_OUTLOOK_DRAFTS
 *
 * These tags are at the least valid for the default message store.
 */
extern BRUTUS_BRESULT brutus_get_outlook_folder(BRUTUS_IMsgStore MsgStore,
						const BRUTUS_BDEFINE PropTag,
						BRUTUS_IMAPIFolder *Folder,
						CORBA_Environment *ev);

/*
 * Will return a string listing the values of the ListBy property tag
 * for all objects that are returned when GetContentsTable() are
 * invoked on Folder.
 */
extern char *brutus_list_folder(BRUTUS_IMAPIFolder Folder,
				const BRUTUS_BDEFINE ListBy,
				CORBA_Environment *ev);

/*
 * Will return a string listing the values of all properties
 * for the object.
 */
extern char *brutus_print_properties(BRUTUS_IMAPIProp PropObject,
				     CORBA_Environment *ev);

/* 
 * Will ping the server. Returns FALSE if it can't do that,
 * TRUE otherwise.
 */
extern gboolean brutus_server_ping(const char *brutus_server);

/*
 * Returns FALSE if definitely disconnected, TRUE is likely connected
 */
extern gboolean brutus_is_connected(const char *brutus_server,
				    BRUTUS_IMAPISession mapi_session);

/*
 * Will open the property with IStream and reads the binary content.
 * Returns NULL for errors or if there is no data to be read. The return 
 * value must be freed by callee.
 */
extern BRUTUS_seq_octet* brutus_read_binary_stream(const BRUTUS_BDEFINE property,
						   BRUTUS_IMAPIProp object);

/*
 * Will open the property with IStream and reads the textual content.
 * Returns NULL for errors or if there is no data to be read. The return 
 * value must be freed by callee.
 */
extern char* brutus_read_text_stream(const BRUTUS_BDEFINE property,
				     BRUTUS_IMAPIProp object);

/*
 * Adds an attachment to the IMessage object.
 * Returns TRUE if all is well, FALSE otherwise.
 */
extern gboolean brutus_add_attachment(BRUTUS_IMessage mapi_message,
				      const gboolean default_attach_rendering,
				      const BRUTUS_BRUTUS_ATTACH_TAG attach_tag,
				      const BRUTUS_seq_SPropValue *aditional_props,
				      const CORBA_unsigned_long attach_data_len,
				      const void *attach_data);

/*
 * builds a (NOT[EXIST:SPROP] || (EXIST:SPROP && SPROP[VALUE OPER CONST])) SRestrictionContainer.
 *
 * That is: Matches if the property operator is fulfilled or if 
 * the property doesn't exist at all. This is good for checks on 
 * the last modification time where you want to get all objects 
 * that are modified after a given cut-off time.
 *
 * You also wants all objects that doesn't have any modification time
 * property at all. You want those objects to ensure that changes in
 * these are picked up as well.
 */
extern BRUTUS_SRestrictionContainer brutus_build_property_restriction_time(BRUTUS_RELOP_TYPE operator,
									   BRUTUS_SPropValue pv,
									   PortableServer_POA root_poa,
									   PortableServer_POA content_poa);

G_END_DECLS

#endif // _BRUTUS_MAPI_H_
